#state machine that makes it easier to build an svg path
#handles SVG syntax
import logging

class PathManager:
	def __init__(self):
		self.lastPointX = None;
		self.lastPointY = None;
		self.firstPointX = None;
		self.firstPointY = None;
		self.points = [];
		self.MOVETO = "M ";
		self.LINETO = " L ";
		self.CLOSE = "Z ";
		self.path = "";
		self.pointFormat = "%(x)0.4f %(y)0.4f "
	#end the current path.
	
	 		    		
	def isClose(self, pt1,pt2,tolerance):
		if (not pt1 == None) and (not pt2 == None):
			return abs(pt1 - pt2) <= tolerance
		else:
			return False;
		
	def printPoint(self,x,y):
		return self.pointFormat %  { 'x':x , 'y':y };
	
	#try to close the most recent path.
	#if the last point is the same as the first,
	#then remove the last point and replace with an SVG close command instead
	def endSegment(self):
		if self.isClose(self.lastPointX,self.firstPointX,0.0001) and self.isClose(self.lastPointY,self.firstPointY,0.0001):
			path = self.path.split();
			self.path = " ".join(path[0:len(path)-3]) + " Z";
		self.firstPointX = None;
		self.firstPointY = None;
			
	#add a point
	#this is the main point of the class-- redundant data points
	#are avoided.
	def addPoint ( self,x, y):
		logging.debug( "Adding Point: " + self.printPoint(x,y));
		if self.firstPointX == None:
			#no first point
			self.firstPointX = x;
			self.firstPointY = y;
			self.path += self.MOVETO;
			self.path += self.printPoint(x,y);			
		else:
			if x == self.lastPointX and y == self.lastPointY:
				logging.debug( "Ignoring Redundant point: " + self.printPoint(x,y) );
			else:
				self.path += self.LINETO;		
				#append the point
				self.path += self.printPoint(x,y);			
		
		self.lastPointX = x;
		self.lastPointY = y;

		
	def toSVG(self):
		return self.path;
		
if __name__=='__main__':
	pm = PathManager();
	pm.addPoint(1,2);
	pm.addPoint(1,2);
	pm.addPoint(3,4);
	pm.addPoint(3,4);
	pm.addPoint(5,6);
	pm.addPoint(1,2);
	pm.endSegment();
	print pm.toSVG(),"<-- should be M 1.000 2.000 L 3.000 4.000 L5.000 6.000 Z";
	pm.addPoint (0.1,0.1);
	pm.addPoint (0.2,0.2);
	pm.addPoint (0.3,0.3);
	pm.addPoint (0.3,0.3);
	pm.endSegment();
	print pm.toSVG(),"<-- should be M 1.000 2.000 L 3.000 4.000 L5.000 6.000 Z M0.100 0.100 L0.200 0.200 L0.300 0.300";
	
